#!/bin/sh
#set -x

if test $# -ne 5; then
    echo "Usage: $0 <joincmd> <recipe_name> <rbs_rb_working_directory> <product_directory> <product_name_prefix>" >&2
    exit 1
fi


# QCLogWriter configuration

instrument=MUSE
vlt_dictionaries=PRIMARY-FITS,OBS,DPR,TPL,PRO,NGCCON,NGCDCS,MUSE_ICS,MUSE_OS,DETMON_QC
paf_pattern='.*\(qc_\).*\.paf'


# Directory paths 

rbs_rb_working_directory=$3
product_directory=$4
olas_directory=${DFS_DATA_REDUCED_OLAS}


# Post-processing tools

joincmd=$1


# Recipe and products

recipe_name=$2
product_name_prefix=$5



if test ! -x ${joincmd}; then
    echo "ERROR: $0: joincmd=${joincmd} does not exist or is not executable" >&2
    exit 1
fi

if test ! -d ${rbs_rb_working_directory}; then
    echo "ERROR: $0: rbs_rb_working_directory=${rbs_rb_working_directory} does not exist or is no directory" >&2
    exit 1
fi

if test ! -d ${product_directory}; then
    echo "ERROR: $0: product_directory=${product_directory} does not exist or is no directory" >&2
    exit 1
fi


# Define recipe product map.
#
# This needs to be kept in sync with the
# actual recipe products!

case $recipe_name in
muse_ampl)
    multi_product_id=""
    single_product_id=""
    final_product_id="AMPL_CONVOLVED"
    ;;
muse_lingain)
    multi_product_id=""
    single_product_id=""
    final_product_id="NONLINEARITY_GAIN"
    ;;
muse_bias)
    multi_product_id=""
    single_product_id=""
    final_product_id="MASTER_BIAS"
    ;;
muse_dark)
    multi_product_id=""
    single_product_id=""
    final_product_id="MASTER_DARK"
    ;;
muse_flat)
    multi_product_id=""
    single_product_id=""
    final_product_id="MASTER_FLAT TRACE_TABLE"
    ;;
muse_wavecal)
    multi_product_id=""
    single_product_id=""
    final_product_id="ARC_RED_LAMP_0001 ARC_RED_LAMP_0002 ARC_RED_LAMP_0003 WAVECAL_TABLE"
    ;;
muse_lsf)
    multi_product_id=""
    single_product_id=""
    final_product_id="LSF_PROFILE"
    ;;
muse_geometry)
    multi_product_id=""
    single_product_id=""
    final_product_id="GEOMETRY_TABLE"
    ;;
muse_twilight)
    multi_product_id=""
    single_product_id=""
    final_product_id="DATACUBE_SKYFLAT TWILIGHT_CUBE"
    ;;
muse_scibasic)
    # Note: The sequence (exposure) number is hardcoded
    # here (0001) since the on-line pipeline is configured
    # to process each exposure individually!

    multi_product_id="OBJECT_RED_0001 STD_RED_0001 ASTROMETRY_RED_0001"
    single_product_id="PIXTABLE_OBJECT_0001" 
    single_product_id="${single_product_id} PIXTABLE_STD_0001"
    single_product_id="${single_product_id} PIXTABLE_SKY_0001"
    single_product_id="${single_product_id} PIXTABLE_ASTROMETRY_0001"
    final_product_id=""

    intermediate_product_id="${single_product_id}"
    ;;
muse_standard)
    multi_product_id=""
    single_product_id=""
    final_product_id="DATACUBE_STD_0001 STD_RESPONSE_0001 STD_TELLURIC_0001 STD_FLUXES_0001"
    ;;
muse_create_sky)
    multi_product_id=""
    single_product_id=""
    final_product_id="SKY_MASK SKY_SPECTRUM SKY_LINES SKY_CONTINUUM IMAGE_FOV"
    ;;
muse_astrometry)
    multi_product_id=""
    single_product_id=""
    final_product_id="DATACUBE_ASTROMETRY_0001 ASTROMETRY_WCS_0001"
    ;;
muse_scipost)
    multi_product_id=""
    single_product_id=""
    final_product_id="DATACUBE_FINAL IMAGE_FOV_0001 IMAGE_FOV_0002 IMAGE_FOV_0003 IMAGE_FOV_0004"
    ;;
detmon_opt_lg_mr)
    multi_product_id=""
    single_product_id=""
    final_product_id="detmon_opt_lg_mr_linearity_table detmon_opt_lg_mr_gain_table"
    ;;
*)
   echo "ERROR: $0: recipe=${recipe_name} is not supported (product configuration is missing)!" >&2
   exit 1
   ;;
esac


# If required, join partial products, and create the 
# symbolic links for the joined product in the OLAS
# directory. 

result=0
product_index=0
recipe_products=""
intermediate_products=""


if test ${result} -eq 0 -a -n "${multi_product_id}"; then

    for product_prefix in ${multi_product_id}; do

        product_name_suffix="`printf '%04d' $product_index`"
        product_path=${product_directory}/${product_name_prefix}_${product_name_suffix}.fits
        product_parts=`find ${rbs_rb_working_directory}/MainJob-1 -name "${product_prefix}-[0-9][0-9].fits" | sort`

        if test -z "${product_parts}"; then
            echo "WARNING: $0: cannot find any ${product_prefix} product parts to join." >&2
        else

            if test -e ${product_path}; then
                echo "WARNING: $0: removing already-existing product ${product_path}" >&2
                rm -f ${product_path}
            fi

            eval ${joincmd} --pipefile ${product_prefix}.fits ${product_parts} ${product_path}

            if test $? -ne 0; then
                echo "ERROR: $0: joining ${product_prefix} product parts failed!" >&2 
                echo "ERROR: $0: failed to create product ${product_path}" >&2
                result=1
                break;
            else
                recipe_products="${recipe_products} ${product_path}"


                # If required re-ingest selected products into the DO raw data stream

                echo ${intermediate_product_id} | grep ${product_prefix} >/dev/null

                if test $? -eq 0; then
                    intermediate_products="${intermediate_products} ${product_path}"
                fi
            fi

            product_index=$((product_index + 1))
        fi

    done

fi


if test ${result} -eq 0 -a -n "${single_product_id}"; then

    for product_prefix in ${single_product_id}; do

        product_list=`find ${rbs_rb_working_directory}/MainJob-1 -name "${product_prefix}-[0-9][0-9].fits" | sort`

        if test -z "${product_list}"; then
            echo "WARNING: $0: cannot find any ${product_prefix} products to process." >&2
        else

            for product_file in ${product_list}; do

                product_name_suffix="`printf '%04d' $product_index`"
                product_path=${product_directory}/${product_name_prefix}_${product_name_suffix}.fits

                if test -e ${product_path}; then
                    echo "WARNING: $0: removing already-existing product ${product_path}" >&2
                    rm -f ${product_path}
                fi

                eval mv ${product_file} ${product_path}

                if test $? -ne 0; then
                    echo "ERROR: $0: moving ${product_prefix} products failed!" >&2 
                    echo "ERROR: $0: failed to create product ${product_path}" >&2
                    result=1
                    break;
                else
                    recipe_products="${recipe_products} ${product_path}"


                    # If required re-ingest selected products into the DO raw data stream

                    echo ${intermediate_product_id} | grep ${product_prefix} >/dev/null

                    if test $? -eq 0; then
                        intermediate_products="${intermediate_products} ${product_path}"
                    fi
                fi

                product_index=$((product_index + 1))

            done
        fi

    done

fi

if test ${result} -eq 0 -a -n "${final_product_id}"; then

    for product_prefix in ${final_product_id}; do

        product_list=`find ${rbs_rb_working_directory}/MainJob-1 -name "${product_prefix}.fits" | sort`

        if test -z "${product_list}"; then
            echo "WARNING: $0: cannot find any ${product_prefix} products to process." >&2
        else

            for product_file in ${product_list}; do

                product_name_suffix="`printf '%04d' $product_index`"
                product_path=${product_directory}/${product_name_prefix}_${product_name_suffix}.fits

                if test -e ${product_path}; then
                    echo "WARNING: $0: removing already-existing product ${product_path}" >&2
                    rm -f ${product_path}
                fi

                eval mv ${product_file} ${product_path}

                if test $? -ne 0; then
                    echo "ERROR: $0: moving ${product_prefix} products failed!" >&2 
                    echo "ERROR: $0: failed to create product ${product_path}" >&2
                    result=1
                    break;
                else
                    recipe_products="${recipe_products} ${product_path}"


                    # If required re-ingest selected products into the DO raw data stream

                    echo ${intermediate_product_id} | grep ${product_prefix} >/dev/null

                    if test $? -eq 0; then
                        intermediate_products="${intermediate_products} ${product_path}"
                    fi
                fi

                product_index=$((product_index + 1))

            done
        fi

    done

fi


# Create symbolic links for OLAS

# Always create the symbolic links as a last step
# to avoid that OLAS transfers an incomplete
# set of reduced products

if test ${result} -eq 0 ; then

    for product in ${recipe_products}; do

        olas_link="${olas_directory}/`basename ${product}`"
        RBSLink ${product} ${olas_link}

        if test $? -ne 0; then
            echo "ERROR: $0: RBSLink failed to create the symbolic link ${olas_link}" >&2
            result=1
            break
        fi

    done

fi

if test ${result} -ne 0; then
    echo "ERROR: $0: execution incomplete due to an unrecoverable problem!" >&2
    exit ${result}
fi


# Write QC logfile

paf_files=`find ${rbs_rb_working_directory} -regex ${paf_pattern}`

if test -n "$paf_files"; then
    paf_files=`echo $paf_files | xargs -n 1 basename | sort`
fi

for paf in ${paf_files}; do

    paf_file=`find ${rbs_rb_working_directory} -name "${paf}"`

    ( cd `dirname ${paf_file}`; \
        QC1LogWriter -s MUSE \
                     -I ${DFS_HOME}/data/SYSTEM/Dictionary \
                     -D ${vlt_dictionaries} \
                     -p `basename ${paf_file}` \
    ) >&2
  
    if test $? -ne 0 ; then
        echo "ERROR: $0: QC1LogWriter failed on ${paf_file}" >&2
        result=1
        break
    fi
  
done


# Re-ingest intermediate products into the DO raw data stream for
# further processing.
#
# This is done as the very last step to make sure that if only
# the link creation fails for some reason, the created products
# will have been processed completely, and no information is missing
# due to a failing product re-ingestion.

if test ${result} -eq 0 -a -n "${intermediate_products}"; then

    for product in ${intermediate_products}; do
        linkToPipeline ${product}

        if test $? -ne 0; then
            echo "ERROR: $0: linkToPipeline failed to create the symbolic link for ${product}" >&2
            result=1
            break
        fi

    done

fi

exit ${result}
